package main.scala.domain

import java.io._
import java.text.Collator
import java.util.Locale

import scala.io.Source

class Universidade() {
    var docentes: Map[Int, Docente] = Map[Int, Docente]()
    var disciplinas: List[Disciplina] = List[Disciplina]()

    def leOrientPosGraduacao(caminho: String): Unit = {
        val file = Source.fromFile(caminho)
        file.getLines().drop(1).foreach(linha => {
            val items: Array[String] = linha.split(";").map(_.trim)
            val codigoDocente = items(0).toInt
            val cargaSemanal = items(4).toInt
            docentes(codigoDocente).horasSemanaisOrient += cargaSemanal
        })
        file.close()
    }

    def leOrientGraduacao(caminho: String): Unit = {
        val file = Source.fromFile(caminho)
        file.getLines().drop(1).foreach(linha => {
            val items: Array[String] = linha.split(";").map(_.trim)
            val codigoDocente = items(0).toInt
            val cargaSemanal = items(3).toInt
            docentes(codigoDocente).horasSemanaisOrient += cargaSemanal
        })
        file.close()
    }

    def leDisciplinas(caminho: String): Unit = {
        val file = Source.fromFile(caminho)
        disciplinas = file.getLines().drop(1).map(linha => {
            val items: Array[String] = linha.split(";").map(_.trim)
            val codigoDisciplina = items(0)
            val nomeDisciplina = items(1)
            val codigoDocente = items(2).toInt
            val cargaSemanal = items(3).toInt
            val cargaSemestral = items(4).toInt
            docentes(codigoDocente).horasSemanaisAulas += cargaSemanal
            docentes(codigoDocente).horasSemestraisAulas += cargaSemestral

            new Disciplina(docentes(codigoDocente), codigoDisciplina, nomeDisciplina, cargaSemestral)
        }).toList
        file.close()
    }

    def leProducoes(caminho: String): Unit = {
        val file = Source.fromFile(caminho)
        file.getLines().drop(1).foreach(linha => {
            val items: Array[String] = linha.split(";").map(_.trim)
            val codigo = items(0).toInt
            //            val nome = items(1)
            val qualif = items.length == 3 && items(2) == "X"
            docentes(codigo).addPaper(qualif)
        })
        file.close()
    }

    def leDocentes(caminho: String): Unit = {
        val file = Source.fromFile(caminho)
        docentes = file.getLines().drop(1).map(linha => {
            val items: Array[String] = linha.split(";").map(_.trim)
            val codigo = items(0).toInt
            val nome = items(1)
            val dept = items(2)
            codigo -> new Docente(codigo, nome, dept)
        }).toMap
    }

    def escrevePAD(caminhoPAD: String): Unit = {
        val ord = Ordering
          .comparatorToOrdering(Collator.getInstance(Locale.getDefault))
          .asInstanceOf[Ordering[String]]
        val listaDocentes = docentes.values.toList
        val saida = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(caminhoPAD), "utf-8"))
        saida.write("Docente;Departamento;Horas Semanais Aula;Horas Semestrais Aula;Horas Semanais Orientação;Produções Qualificadas;Produções Não Qualificadas\n")
        listaDocentes.sortBy(_.nome)(ord).map(docente => {
            var linha = s"${docente.nome};"
            linha += s"${docente.departamento};"
            linha += s"${docente.horasSemanaisAulas};"
            linha += s"${docente.horasSemestraisAulas};"
            linha += s"${docente.horasSemanaisOrient};"
            linha += s"${docente.papersQualif};"
            linha += s"${docente.papersNaoQualif}"
            linha + "\n"
        }).foreach(saida.write)
        saida.close()
    }

    def escreveAlocacao(caminho: String): Unit = {
        val saida = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(caminho), "utf-8"))
        saida.write("Docente;Código;Nome;Carga Horária Semestral\n")
        disciplinas.sorted.map(disciplina => {
            var linha = s"${disciplina.docente.nome};"
            linha += s"${disciplina.codigo};"
            linha += s"${disciplina.nome};"
            linha += s"${disciplina.cargaSemestral}"
            linha + "\n"
        }).foreach(saida.write)
        saida.close()
    }
}
